'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var packageName = 'subscribeWhileMounted';

var string = function string(value) {
    return typeof value === 'string' && !!value;
};
var func = function func(value) {
    return typeof value === 'function';
};
var undef = function undef(value) {
    return typeof value === 'undefined';
};
var nonEmptyArrayOf = function nonEmptyArrayOf(validator) {
    return function (array) {
        return Array.isArray(array) && array.length !== 0 && array.every(function (v) {
            return validator(v);
        });
    };
};
var oneOfType = function oneOfType(possibleTypes) {
    return function (value) {
        return possibleTypes.some(function (type) {
            return type(value);
        });
    };
};

var validReactComponent = function validReactComponent(comp) {
    return comp && func(comp.setState) && oneOfType([undef, func])(comp.componentWillUnmount);
};
var validProps = oneOfType([undef, string, nonEmptyArrayOf(string)]);
var validCallback = oneOfType([undef, func]);

module.exports = function (store) {
    return function (reactComp, oneOrMoreProps, optionalCallback) {
        // Validate input
        if (!validReactComponent(reactComp)) {
            throw new Error(packageName + ' received a first argument of type \'' + (typeof reactComp === 'undefined' ? 'undefined' : _typeof(reactComp)) + '\'. First argument must be a React Component.');
        }
        if (!validProps(oneOrMoreProps)) {
            throw new Error(packageName + ' received a second argument of type \'' + (typeof oneOrMoreProps === 'undefined' ? 'undefined' : _typeof(oneOrMoreProps)) + '\'. Second argument must be a string, an array of strings or undefined.');
        }
        if (!validCallback(optionalCallback)) {
            throw new Error(packageName + ' received a third argument of type \'' + (typeof optionalCallback === 'undefined' ? 'undefined' : _typeof(optionalCallback)) + '\'. Third argument must be a function or undefinded.');
        }

        // Decide properties
        var properties = Array.isArray(oneOrMoreProps) && oneOrMoreProps || typeof oneOrMoreProps === 'string' && [oneOrMoreProps] || typeof oneOrMoreProps === 'undefined' && false;

        // Check that all properties to subscribe to exist in store's state
        var initStoreState = store.getState();
        var propIsMissing = function propIsMissing(prop) {
            return _typeof(initStoreState[prop]) !== 'object';
        };

        if (properties && properties.some(propIsMissing)) {
            var missingProps = properties.filter(propIsMissing);
            var single = missingProps.length === 1;

            throw new Error(packageName + ' can only subscribe to changes from the store\'s reducers. ' + ('\'' + missingProps.join('\' and \'') + '\' ' + (single ? 'is not an object' : 'are not objects') + ' in the store\'s initial state.'));
        }

        // Decide callback
        var callback = typeof optionalCallback === 'function' ? optionalCallback : reactComp.setState.bind(reactComp);

        // Notifier function
        var notifyOfUpdates = properties ? getMethodToNotifyOnUpdatesToSelectedProperties(store, properties, callback, !!optionalCallback) : getMethodToNotifyOnAnyUpdates(store, callback);

        // Notify callback once and on each update
        notifyOfUpdates();
        var unsubscribe = store.subscribe(notifyOfUpdates);

        // Automatically un-subscribe before un-mounting
        reactComp.componentWillUnmount = getNewComponentWillUnmount(unsubscribe, reactComp.componentWillUnmount);
    };
};

function getMethodToNotifyOnUpdatesToSelectedProperties(store, properties, callback, passEntireStateAsSecondArgument) {
    var lastStates = {};

    return function () {
        var storeState = store.getState();

        var updates = Object.keys(storeState).filter(function (key) {
            return properties.indexOf(key) !== -1;
        }).filter(function (key) {
            var isUpdated = lastStates[key] !== storeState[key];

            lastStates[key] = storeState[key];

            return isUpdated;
        }).reduce(function (obj, key) {
            return _extends({}, obj, _defineProperty({}, key, storeState[key]));
        }, {});

        if (Object.keys(updates).length !== 0) {
            if (passEntireStateAsSecondArgument) {
                callback(updates, storeState);
            } else {
                callback(updates);
            }
        }
    };
}

var getMethodToNotifyOnAnyUpdates = function getMethodToNotifyOnAnyUpdates(store, callback) {
    return function () {
        return callback(store.getState());
    };
};

var getNewComponentWillUnmount = function getNewComponentWillUnmount(unsubscribe, componentWillUnmount) {
    return function () {
        unsubscribe();
        componentWillUnmount && componentWillUnmount();
    };
};